"""
    Except.ExceptionTypes.py

    This module defines custom exception types for the MOLASS library.
"""

class NotSpecifedError(Exception) : pass
class InadequateUseError(Exception) : pass
class InconsistentUseError(Exception): pass