"""
    Batch.V1Result.py

    This module contains the class V1Result, which is used to store the results of the V1 batch process.

    Copyright (c) 2025, SAXS Team, KEK-PF 
"""

class V1Result:
    """
    A class to represent the results of the V1 batch process.
    """

    def __init__(self):
        """
        """
        pass